#  Copyright (c) 2024 Mira Geoscience Ltd.
#
#  This file is part of edge-detection package.
#
#  All rights reserved.
#
#
#  This file is part of curve-apps.
#
#  curve-apps is distributed under the terms and conditions of the MIT License
#  (see LICENSE file at the root of this source code package).

# pylint: disable=duplicate-code

from __future__ import annotations

from typing import Optional, Union

from geoapps_utils.driver.data import BaseData
from geoapps_utils.numerical import DetectionParameters
from geoh5py.data import Data, ReferencedData
from geoh5py.objects import Curve, Points
from geoh5py.ui_json import InputFile
from pydantic import BaseModel, ConfigDict

from curve_apps import assets_path

from ..params import OutputParameters

NAME = "trend_lines"
DEFAULT_UI_JSON = assets_path() / f"uijson/{NAME}.ui.json"


class SourceParameters(BaseModel):
    """
    Source parameters expected by the ui.json file format.

    :param entity: A Curve or Points source object.
    :param data: Data values to find edges on.
    :param parts: Optional parts to connect.
    """

    model_config = ConfigDict(arbitrary_types_allowed=True)

    entity: Union[Curve, Points]
    data: Optional[ReferencedData] = None
    parts: Optional[Data] = None


class Parameters(BaseData):
    """
    Parts connection input parameters.

    :param detection: Detection parameters expected for the parts connection.
    :param source: Parameters for the source object and data parameters.
    """

    model_config = ConfigDict(arbitrary_types_allowed=True)

    _name: str = NAME

    detection: DetectionParameters
    input_file: Optional[InputFile] = InputFile.read_ui_json(
        DEFAULT_UI_JSON, validate=False
    )
    output: OutputParameters
    run_command: str = f"curve_apps.{NAME}.driver"
    source: SourceParameters
    title: str = NAME.capitalize().replace("_", " ")



#  Copyright (c) 2024 Mira Geoscience Ltd.
#
#  This file is part of curve-apps package.
#
#  All rights reserved.

from __future__ import annotations

from pathlib import Path

__version__ = "0.1.0-alpha.1"


def assets_path() -> Path:
    """Return the path to the assets folder."""

    parent = Path(__file__).parent
    folder_name = f"{parent.name}-assets"
    assets_folder = parent.parent / folder_name
    if not assets_folder.is_dir():
        raise RuntimeError(f"Assets folder not found: {assets_folder}")

    return assets_folder



#  Copyright (c) 2024 Mira Geoscience Ltd.
#
#  This file is part of edge-detection package.
#
#  All rights reserved.
#
#
#  This file is part of curve-apps.
#
#  curve-apps is distributed under the terms and conditions of the MIT License
#  (see LICENSE file at the root of this source code package).

from __future__ import annotations

from typing import Optional

from geoapps_utils.driver.data import BaseData
from geoh5py.data import FloatData
from geoh5py.objects import Grid2D
from geoh5py.ui_json import InputFile
from pydantic import BaseModel, ConfigDict

from curve_apps import assets_path

from ..params import OutputParameters

NAME = "edge_detection"
DEFAULT_UI_JSON = assets_path() / f"uijson/{NAME}.ui.json"


class SourceParameters(BaseModel):
    """
    Source parameters expected by the ui.json file format.

    :param objects: A Grid2D source object.
    :param data: Data values to find edges on.
    """

    model_config = ConfigDict(arbitrary_types_allowed=True)

    objects: Grid2D
    data: FloatData


class DetectionParameters(BaseModel):
    """
    Detection parameters expected by the ui.json file format.

    :param line_length: Minimum accepted pixel length of detected lines. (Hough)
    :param line_gap: Maximum gap between pixels to still form a line. (Hough)
    :param sigma: Standard deviation of the Gaussian filter. (Canny)
    :param threshold: Value threshold. (Hough)
    :param window_size: Size of the window to search for lines.
    :param merge_length: Minimum length between nodes that should be merged.
    """

    line_length: int = 1
    line_gap: int = 1
    sigma: float = 10
    threshold: int = 1
    window_size: Optional[int] = None
    merge_length: Optional[int] = None


class Parameters(BaseData):
    """
    Edge detection input parameters.

    :param detection: Detection parameters expected for the edge detection.
    :param source: Parameters for the source object and data.
    """

    model_config = ConfigDict(arbitrary_types_allowed=True)

    _name: str = NAME

    input_file: Optional[InputFile] = InputFile.read_ui_json(
        DEFAULT_UI_JSON, validate=False
    )
    detection: DetectionParameters
    output: OutputParameters
    run_command: str = f"curve_apps.{NAME}.driver"
    source: SourceParameters
    title: str = NAME.capitalize().replace("_", " ")



#  Copyright (c) 2024 Mira Geoscience Ltd.
#
#  This file is part of edge-detection package.
#
#  All rights reserved.
#
#
#  This file is part of curve-apps.
#
#  curve-apps is distributed under the terms and conditions of the MIT License
#  (see LICENSE file at the root of this source code package).


from __future__ import annotations

import logging
import tempfile
from abc import abstractmethod

from geoapps_utils.driver.data import BaseData
from geoapps_utils.driver.driver import BaseDriver
from geoh5py.groups import ContainerGroup
from geoh5py.objects import ObjectBase
from geoh5py.ui_json import InputFile

logger = logging.getLogger(__name__)


class BaseCurveDriver(BaseDriver):
    """
    Driver for the edge detection application.

    :param parameters: Application parameters.
    """

    _parameter_class: type[BaseData]
    _default_name: str

    def __init__(self, parameters: BaseData | InputFile):
        if isinstance(parameters, InputFile):
            parameters = self._parameter_class.build(parameters)

        # TODO need to re-type params in base class
        super().__init__(parameters)

    def run(self):
        """
        Run method of the driver.
        """
        with self.workspace.open(mode="r+") as workspace:
            parent = None
            if self.params.output.out_group is not None:
                parent = ContainerGroup.create(
                    workspace=workspace,
                    name=self.params.output.out_group,
                )

            name = self.params.output.export_as
            if name is None:
                name = self._default_name

            logger.info("Begin process.")
            output = self.create_output(name, parent=parent)
            logger.info("Process completed.")

            if output is not None:
                self.update_monitoring_directory(
                    parent if parent is not None else output
                )

            logger.info("Curve object '%s' saved to '%s'.", name, str(workspace.h5file))

    @abstractmethod
    def create_output(self, name: str, parent: ContainerGroup | None = None):
        """
        Create output.
        """

    @property
    def params(self) -> BaseData:
        """Application parameters."""
        return self._params

    @params.setter
    def params(self, val: BaseData):
        if not isinstance(val, BaseData):
            raise TypeError("Parameters must be of type Parameters.")
        self._params = val

    def add_ui_json(self, entity: ObjectBase | ContainerGroup):
        """
        Add ui.json file to entity.

        :param entity: Object to add ui.json file to.
        """
        param_dict = self.params.flatten()
        self.params.input_file.update_ui_values(param_dict)

        with tempfile.TemporaryDirectory() as temp_dir:
            file_path = self.params.input_file.write_ui_json(
                path=temp_dir, name=self.params.name + ".ui.json"
            )
            entity.add_file(str(file_path))



#  Copyright (c) 2024 Mira Geoscience Ltd.
#
#  This file is part of edge-detection package.
#
#  All rights reserved.
#
#
#  This file is part of curve-apps.
#
#  curve-apps is distributed under the terms and conditions of the MIT License
#  (see LICENSE file at the root of this source code package).

from __future__ import annotations

from typing import Optional

from pydantic import BaseModel


class OutputParameters(BaseModel):
    """
    Output parameters expected by the ui.json file format.

    :param export_as: Name of the output entity.
    :param out_group: Name of the output group.
    """

    export_as: Optional[str] = None
    out_group: Optional[str] = None
